// 方向キー（キーボード／タッチ）コントローラーオブジェクト
var m11RPGwj_Controller = function(type) {

	// タッチ状態管理オブジェクト
	this._touchController = null;

	// キーボード状態管理オブジェクト
	this._keyController = null;

	// タッチ画面上サイズ
	this._width = 0;
	this._height = 0;

	// タッチ対応
	if ((type & m11RPGwj_Controller.TOUCH_TYPE) != 0) {

		this._touchController = new m11RPGwj_TouchController(this);

		this._width = this._touchController.getWidth();
		this._height = this._touchController.getHeight();

	}

	// キーボード対応
	if ((type & m11RPGwj_Controller.KEYBOARD_TYPE) != 0) {
		this._keyController = new m11RPGwj_KeyController();
	}

	this.getWidth = function() {
		return this._width;
	}

	this.getHeight = function() {
		return this._height;
	}

	this.getX = function() {

		if (this._touchController == null) {
			return 0;
		} else {
			return this._touchController.getX();
		}

	}

	this.getY = function() {

		if (this._touchController == null) {
			return 0;
		} else {
			return this._touchController.getY();
		}

	}

	this.setPos = function(x, y) {
		if (this._touchController != null) {
			this._touchController.setPos(x, y);
		}
	}

	this.setParent = function(cv) {
		if (this._touchController != null) {
			this._touchController.setParent(cv);
		}
	}

	// キー状態を取得
	this.getKeyStatus = function() {

		var key = m11RPGwj_Controller.KEYFLAG_NONE;

		// タッチ操作の方向キー状態取得
		if (this._touchController != null) {
			key |= this._touchController.getKeyStatus();
		}

		// キーボード操作の方向キー状態取得
		if (this._keyController != null) {
			key |= this._keyController.getKeyStatus();
		}

		// タッチ操作・キーボード操作の和を現在のキー状態として返す
		return key;

	}

	this.getStick = function() {

		var stick = m11RPGwj_Controller.KEYFLAG_NONE;

		// タッチ操作の方向キー状態取得
		if (this._touchController != null) {
			stick |= this._touchController.getStick();
		}

		// キーボード操作の方向キー状態取得
		if (this._keyController != null) {
			stick |= this._keyController.getStick();
		}

		// タッチ操作・キーボード操作の和を現在の方向キー状態として返す
		return stick;

	}

	this.getButtons = function() {

		var buttons = m11RPGwj_Controller.KEYFLAG_NONE;

		if (this._touchController != null) {
			buttons |= this._touchController.getButtons();
		}

		if (this._keyController != null) {
			buttons |= this._keyController.getButtons();
		}

		return buttons;

	}

	this.eventHandler = function(e) {

		if (this._touchController != null) {
			this._touchController.eventHandler(e);
		}

		if (this._keyController != null) {
			this._keyController.eventHandler(e);
		}

	}

	this.draw = function() {
		if (this._touchController != null) {
			this._touchController.draw();
		}
	}

}

// コントローラーオブジェクト作成
m11RPGwj_Controller.createController = function(type) {
	m11RPGwj_Controller.controller = new m11RPGwj_Controller(type);
}

m11RPGwj_Controller.controller = null;

m11RPGwj_Controller.KEYFLAG_NONE = 0;

m11RPGwj_Controller.KEYFLAG_UP = 1;
m11RPGwj_Controller.KEYFLAG_RIGHT = 2;
m11RPGwj_Controller.KEYFLAG_DOWN = 4;
m11RPGwj_Controller.KEYFLAG_LEFT = 8;
m11RPGwj_Controller.KEYFLAG_SELECT = 16;
m11RPGwj_Controller.KEYFLAG_CANCEL = 32;

m11RPGwj_Controller.STICK_MASK = 15;
m11RPGwj_Controller.BUTTONS_MASK = 48;

m11RPGwj_Controller.KEYBOARD_TYPE = 1;
m11RPGwj_Controller.TOUCH_TYPE = 2;

