// メッセージ表示ウインドウコンストラクタ
var m11RPGwj_MessageWindow = function() {

	m11RPGwj_Window.call(this);

	// 枠内余白
	this._content_padding = m11RPGwj_MessageWindow.DEFAULT_CONTENT_PADDING;

	// 文字列表示領域左上座標
	this._content_x = 0;
	this._content_y = 0;

	// 文字列表示領域の大きさ
	this._content_width = 0;
	this._content_height = 0;

	// 1ページの表示行数
	this._page_lines = 0;

	// 文字列描画フォント
	this._font = m11RPGwj_MessageWindow.DEFAULT_FONT;

	// 一行の高さ
	this._line_height = m11RPGwj_MessageWindow.DEFAULT_LINE_HEIGHT;

	// 表示文字列（ページ配列）
	this._pageList = new Array();

	this._process_stage = 0;
	this._process_count = 0;

	// 現在ページ
	this._page_index = 0;

	// 現在表示位置
	this._putting_line = 0;
	this._putting_pos = 0;

	// ウインドウの大きさを設定
	this.setSize = function(width, height) {

		m11RPGwj_Window.prototype.setSize.call(this, width, height);

		// 文字列表領域の大きさを計算
		this._content_width = this._client_width - (this._content_padding * 2);
		this._content_height = this._client_height - (this._content_padding * 2);

		// 1ページあたりの行数を計算
		this._page_lines = Math.floor(this._content_height / this._line_height);

	}

	// 表示位置設定
	this.setPos = function(x, y) {

		m11RPGwj_Window.prototype.setPos.call(this, x, y);

		this._content_x = this._client_x + this._content_padding;
		this._content_y = this._client_y + this._content_padding;

	}

	// ページ全体を表示
	this.pageDone = function() {

		if (this._process_stage == m11RPGwj_MessageWindow.PROCESS_STAGE_PUTTING) {
			this._process_stage = m11RPGwj_MessageWindow.PROCESS_STAGE_PAGE_END;
		}

	}

	// 表示文字列を設定
	this.setText = function(text, context) {

		// 表示文字列配列を初期化
		this._pageList = new Array();

		var line = '';
		var lineList = new Array();

		// 文字列処理位置
		var pos = 0;

		var page_flag;

		// フォント設定
		context.font = this._font;

		// ウインドウの横幅に合わせて各行のテキスト作成
		while (pos < text.length) {

			// 改ページフラグリセット
			page_flag = false;

			// 文字列の描画幅がウインドウ幅を超えるまで現在行に文字追加
			while (pos < text.length && context.measureText(line + text.charAt(pos)).width < this._content_width) {

				// 現在位置に<:br>タグがあれば改行し現在位置をタグの次の文字へ
				if (text.indexOf('<:br>', pos) == pos) {

					pos += 5;

					// 現在行の処理を終了する
					break;

				}

				// 現在位置に<:page>タグがあれば改ページ処理へ
				if (text.indexOf('<:page>', pos) == pos) {

					pos += 7;

					// 改ページフラグセット
					page_flag = true;

					// 現在行の処理を終了する
					break;

				}

				// 現在行に現在位置の文字を追加
				line = line + text.charAt(pos);

				// 現在位置更新
				pos++;

			}

			// 現在行の文字列を行配列に追加
			lineList.push(line);

			// 改ページフラグがセットされていたら改ページ文字を挿入
			if (page_flag) {
				lineList.push("\f");
			}

			// 現在行の文字列を初期化
			line = '';

		}

		// 逐次出力中の文字列位置
		var page_pos = 0;
		var line_pos = 0;

		var i;

		// 行配列をページ配列に格納
		while (line_pos < lineList.length) {

			// 新ページ作成
			this._pageList[page_pos] = new Array();

			// ページ配列の現在ページに行配列の各行を追加
			for (i = 0;(i < this._page_lines) && (line_pos < lineList.length);i++) {

				// 改ページ文字判定
				if (lineList[line_pos].charAt(0) == "\f") {

					// 現在行位置を改ページ文字行の次へ
					line_pos++;

					// ページ先頭でなければ改ページ処理実行
					if (i > 0) {

						// 現在ページを終了し新規ページへ
						break;

					}

				} else {

					// ページ配列の現在位置に行配列の文字列をコピー
					this._pageList[page_pos].push(lineList[line_pos++]);

				}

			}

			page_pos++;

		}

		// 処理状態を逐次出力に
		this._process_stage = m11RPGwj_MessageWindow.PROCESS_STAGE_PUTTING;

		// 現在ページ位置を最初のページに
		this._page_index = 0;

		// 現在表示位置初期化
		this._putting_line = 0;
		this._putting_pos = 0;

	}

	// 定時処理
	this.process = function(controller) {

		switch (this._process_stage) {

		// ページ開始
		case m11RPGwj_MessageWindow.PROCESS_STAGE_PAGE_START:

			// リターンキー解放待機後にページ表示処理開始
			if (controller.getButtons() == m11RPGwj_Controller.KEYFLAG_NONE) {

				this._putting_line = 0;
				this._putting_pos = 0;

				this._process_stage = m11RPGwj_MessageWindow.PROCESS_STAGE_PUTTING;

			}

			break;

		// ページ表示完了
		case m11RPGwj_MessageWindow.PROCESS_STAGE_PAGE_END:

			// リターンキー解放待機後にページ／メッセージ終了処理へ
			if (controller.getButtons() == m11RPGwj_Controller.KEYFLAG_NONE) {
				if (this._page_index < this._pageList.length - 1) {
					this._process_stage = m11RPGwj_MessageWindow.PROCESS_STAGE_PAGE_EXIT;
				} else {
					this._process_stage = m11RPGwj_MessageWindow.PROCESS_STAGE_MESSAGE_EXIT;
				}
			}

			break;

		// ページ終了処理
		case m11RPGwj_MessageWindow.PROCESS_STAGE_PAGE_EXIT:

			// リターンキーが押されていたら、次ページ開始
			if (controller.getButtons() == m11RPGwj_Controller.KEYFLAG_SELECT) {

				this._page_index++;

				this._process_stage = m11RPGwj_MessageWindow.PROCESS_STAGE_PAGE_START;

			}

			break;

		// メッセージ（全ページ）終了処理
		case m11RPGwj_MessageWindow.PROCESS_STAGE_MESSAGE_EXIT:

			// リターンキーが押されていたら、終了状態へ
			if (controller.getButtons() == m11RPGwj_Controller.KEYFLAG_SELECT) {
				this._process_stage = m11RPGwj_MessageWindow.PROCESS_STAGE_EXIT;
			}

			break;

		// ページ文字列逐次出力中
		case m11RPGwj_MessageWindow.PROCESS_STAGE_PUTTING:

			// 表示位置更新
			this._putting_pos++;

			// 一行の表示終了
			if (this._putting_pos >= this._pageList[this._page_index][this._putting_line].length) {

				// ページ全行の表示完了ならページ終了
				if (this._putting_line == this._pageList[this._page_index].length - 1) {
					this._process_stage = m11RPGwj_MessageWindow.PROCESS_STAGE_PAGE_END;
				} else {

					// 次の行へ
					this._putting_line++;
					this._putting_pos = 0;

				}

			}

			// リターンキーが押されていたらページ全表示
			if (this._process_stage == m11RPGwj_MessageWindow.PROCESS_STAGE_PUTTING && controller.getButtons() == m11RPGwj_Controller.KEYFLAG_SELECT) {
				this._process_stage = m11RPGwj_MessageWindow.PROCESS_STAGE_PAGE_END;
			}

			break;

		}

		this._process_count++;

		// 表示終了状態ならtrue
		return this._process_stage == m11RPGwj_MessageWindow.PROCESS_STAGE_EXIT;

	}

	// メッセージ表示ダイアログ描画関数
	this.draw = function(context) {

		// 可視状態でなければ描画処理は行わない
		if (!this._visible) {
			return;
		}

		m11RPGwj_Window.prototype.draw.call(this, context);

		var i;

		// テキスト描画設定
		context.fillStyle = '#ffffff';
		context.textAlign = 'left';
		context.textBaseline = 'top';

		// フォント設定
		context.font = this._font;

		switch (this._process_stage) {

		// ページ文字列逐次出力中
		case m11RPGwj_MessageWindow.PROCESS_STAGE_PUTTING:

			// 現在行の前まで一気に出力
			for (i = 0;i < this._putting_line;i++) {
				context.fillText(this._pageList[this._page_index][i], this._content_x, this._content_y + i * this._line_height);
			}

			// 現在表示中の行の文字列を表示位置の文字まで描画
			context.fillText(this._pageList[this._page_index][this._putting_line].substring(0, this._putting_pos), this._content_x, this._content_y + i * this._line_height);

			break;

		// ページ全体表示中
		case m11RPGwj_MessageWindow.PROCESS_STAGE_PAGE_END:
		case m11RPGwj_MessageWindow.PROCESS_STAGE_PAGE_EXIT:
		case m11RPGwj_MessageWindow.PROCESS_STAGE_MESSAGE_EXIT:

			// 現在ページ内にある行配列の文字列を描画
			for (i = 0;i < this._pageList[this._page_index].length;i++) {
				context.fillText(this._pageList[this._page_index][i], this._content_x, this._content_y + i * this._line_height);
			}

			break;

		}

		// 文字列描画領域のX方向中心座標を算出
		var content_center_x = this._content_x + (this._content_width / 2);

		// 次ページ待機中なら、次ページマーカーを点滅表示
		if (this._process_stage == m11RPGwj_MessageWindow.PROCESS_STAGE_PAGE_EXIT && (this._process_count % 10) < 5) {

			context.beginPath();

			// ウインドウ下部に三角形のパスを設定
			context.moveTo(content_center_x - 6, this._content_y + this._content_height - 12);
			context.lineTo(content_center_x + 6, this._content_y + this._content_height - 12);
			context.lineTo(content_center_x, this._content_y + this._content_height);

			context.closePath();

			context.fillStyle = '#ffffff';

			// 設定したパスを塗りつぶす
			context.fill();

		}

	}

	this.setSize(m11RPGwj_MessageWindow.DEFAULT_WIDTH, m11RPGwj_MessageWindow.DEFAULT_HEIGHT);

	this.setPos(0, 0);

}

m11RPGwj_MessageWindow.prototype = new m11RPGwj_Window();

m11RPGwj_MessageWindow.DEFAULT_BORDER_SIZE = 1;
m11RPGwj_MessageWindow.DEFAULT_CONTENT_PADDING = 2;
m11RPGwj_MessageWindow.DEFAULT_WIDTH = 224;
m11RPGwj_MessageWindow.DEFAULT_HEIGHT = 160;

m11RPGwj_MessageWindow.DEFAULT_FONT = "14px monospace";
m11RPGwj_MessageWindow.DEFAULT_LINE_HEIGHT = 18;

m11RPGwj_MessageWindow.PROCESS_STAGE_NONE = 0;
m11RPGwj_MessageWindow.PROCESS_STAGE_PUTTING = 1;
m11RPGwj_MessageWindow.PROCESS_STAGE_PAGE_WAIT = 2;
m11RPGwj_MessageWindow.PROCESS_STAGE_PAGE_START = 3;
m11RPGwj_MessageWindow.PROCESS_STAGE_PAGE_END = 4;
m11RPGwj_MessageWindow.PROCESS_STAGE_PAGE_EXIT = 5;
m11RPGwj_MessageWindow.PROCESS_STAGE_MESSAGE_EXIT = 6;
m11RPGwj_MessageWindow.PROCESS_STAGE_EXIT = 7;
