// 選択ダイアログオブジェクト
var m11RPGwj_SelectDialog = function() {

	m11RPGwj_Window.call(this);

	// 枠内余白
	this._content_padding = m11RPGwj_SelectDialog.DEFAULT_CONTENT_PADDING;

	// 選択領域左上座標
	this._content_x = 0;
	this._content_y = 0;

	// 選択領域の大きさ
	this._content_width = 0;
	this._content_height = 0;

	// 文字列描画フォント
	this._font = m11RPGwj_SelectDialog.DEFAULT_FONT;

	// 一行の高さ
	this._item_height = m11RPGwj_SelectDialog.DEFAULT_ITEM_HEIGHT;

	// カーソルキー押下ウエイト
	this._key_wait = 0;

	// 現在の選択位置
	this._index = 0;

	// 可視（描画処理実行）フラグ
	this._visible = false;

	// 選択位置カーソルサイズ
	this._cursor_size = 12;

	// 現在の処理状態
	this._process_stage = m11RPGwj_SelectDialog.PROCESS_STAGE_NONE;

	// 選択肢
	this._itemList = new Array();

	// キャンセル操作可否
	this._cancelable = false;

	// キャンセル操作可否設定
	this.setCancelable = function(arg) {
		this._cancelable = arg;
	}

	// 表示位置設定
	this.setPos = function(x, y) {

		m11RPGwj_Window.prototype.setPos.call(this, x, y);

		this._content_x = this._client_x + this._content_padding;
		this._content_y = this._client_y + this._content_padding;

	}

	// 現在のカーソル（選択）位置を取得
	this.getIndex = function() {
		return this._index;
	}

	this.setIndex = function(index) {
		this._index = index;
	}

	this.getItemNum = function() {

		if (this._itemList == null) {
			return 0;
		}

		return this._itemList.length;

	}

	this.clear = function() {

		this.reset();

		this._itemList = null;

	}

	// 選択肢を設定
	this.setItems = function(items, context) {

		var i;

		this._itemList = new Array();

		var maxWidth = 0;

		// フォント設定
		context.font = this._font;

		// 選択肢文字列中の最大描画幅を取得
		for (i = 0;i < items.length;i++) {

			this._itemList.push(items[i]);

			if (context.measureText(items[i]).width > maxWidth) {
				maxWidth = context.measureText(items[i]).width;
			}

		}

		// ウインドウの大きさを設定
		var client_width = maxWidth + this._cursor_size + (this._content_padding * 2);
		var client_height = items.length * this._item_height + (this._content_padding * 2);

		this.setClientSize(client_width, client_height);

		// 選択肢領域の大きさを計算
		this._content_width = this._client_width - (this._content_padding * 2);
		this._content_height = this._client_height - (this._content_padding * 2);

		// 状態を初期化
		this._key_wait = 0;
		this._index = 0;
		this._process_stage = m11RPGwj_SelectDialog.PROCESS_STAGE_NONE;

	}

	// 選択状態を初期化
	this.reset = function() {

		// 状態を初期化
		this._key_wait = 0;
		this._index = 0;

		this._process_stage = m11RPGwj_SelectDialog.PROCESS_STAGE_NONE;

	}

	// 渡されたキー状態に応じてダイアログ状態を更新
	this.process = function(controller) {

		// キーウエイトが0でなければウエイト処理
		if (this._key_wait > 0) {

			// 方向キー解放状態なら無条件にウエイト終了
			if (controller.getStick() == m11RPGwj_Controller.KEYFLAG_NONE) {
				this._key_wait = 0;
			} else {
				this._key_wait--;
			}

		}

		switch (this._process_stage) {

		// 初期状態
		case m11RPGwj_SelectDialog.PROCESS_STAGE_NONE:

			// トリガー解放を待って選択状態へ
			if (controller.getButtons() == m11RPGwj_Controller.KEYFLAG_NONE) {
				this._process_stage = m11RPGwj_SelectDialog.PROCESS_STAGE_SELECT;
			}

			break;

		// 選択状態
		case m11RPGwj_SelectDialog.PROCESS_STAGE_SELECT:

			// 選択キーが押されたら選択実行
			if (controller.getButtons() == m11RPGwj_Controller.KEYFLAG_SELECT) {
				this._process_stage = m11RPGwj_SelectDialog.PROCESS_STAGE_DONE;
			}

			// キャンセル可能状態でキャンセルキーが押されたらキャンセル状態へ
			if (this._cancelable && (controller.getButtons() == m11RPGwj_Controller.KEYFLAG_CANCEL)) {

				this._index = -1;

				this._process_stage = m11RPGwj_SelectDialog.PROCESS_STAGE_CANCEL;

			}

			// ウエイト中でなければ、上下キー入力処理
			if (this._key_wait == 0) {

				// 上キー処理
				if (controller.getStick() == m11RPGwj_Controller.KEYFLAG_UP) {

					// 現在位置を一つ上（一番上なら一番下）へ
					if (this._index == 0) {
						this._index = this._itemList.length -1;
					} else {
						this._index--;
					}

					// ウエイト開始
					this._key_wait = 10;

				}

				// 下キー
				if (controller.getStick() == m11RPGwj_Controller.KEYFLAG_DOWN) {

					// 現在位置を一つ下（一番下なら一番上）へ
					if (this._index == this._itemList.length -　1) {
						this._index = 0;
					} else {
						this._index++;
					}

					// ウエイト開始
					this._key_wait = 10;

				}

			}

			break;

		// 選択済み状態
		case m11RPGwj_SelectDialog.PROCESS_STAGE_DONE:

			// 選択済みでトリガーキーが解放されたら終了状態としてtrueを返す
			if (controller.getButtons() == m11RPGwj_Controller.KEYFLAG_NONE) {

				return true;

			}

			break;

		// キャンセル済み状態
		case m11RPGwj_SelectDialog.PROCESS_STAGE_CANCEL:

			// 選択済みでトリガーキーが解放されたら終了状態としてtrueを返す
			if (controller.getButtons() == m11RPGwj_Controller.KEYFLAG_NONE) {

				return true;

			}

			break;

		}

		return false;

	}

	this.draw = function(context) {

		if (!this._visible) {
			return;
		}

		m11RPGwj_Window.prototype.draw.call(this, context);

		var i;

		// テキスト描画設定
		context.fillStyle = '#ffffff';
		context.textAlign = 'left';
		context.textBaseline = 'middle';

		// フォント設定
		context.font = this._font;

		// 選択肢文字列を描画
		for (i = 0;i < this._itemList.length;i++) {
			context.fillText(this._itemList[i], this._content_x + this._cursor_size, Math.floor(this._item_height / 2) + this._content_y + i * this._item_height);
		}

		if (this._index >= 0) {

			context.fillStyle = '#ffffff';

			context.beginPath();

			// 現在の選択インデックスの位置に三角形のパスを設定
			context.moveTo(this._content_x + 1, this._content_y + this._index * this._item_height + 2);
			context.lineTo(this._content_x + this._cursor_size - 2, this._index * this._item_height + this._content_y + Math.floor(this._item_height / 2));
			context.lineTo(this._content_x + 1, this._content_y + (this._index + 1) * this._item_height - 2);

			context.closePath();

			// 設定したパスを塗りつぶしカーソルを描画
			context.fill();

		}

	}

}

m11RPGwj_SelectDialog.prototype = new m11RPGwj_Window();

m11RPGwj_SelectDialog.PROCESS_STAGE_NONE = 0;
m11RPGwj_SelectDialog.PROCESS_STAGE_SELECT = 1;
m11RPGwj_SelectDialog.PROCESS_STAGE_DONE = 2;
m11RPGwj_SelectDialog.PROCESS_STAGE_CANCEL = 3;

m11RPGwj_SelectDialog.DEFAULT_BORDER_SIZE = 1;
m11RPGwj_SelectDialog.DEFAULT_CONTENT_PADDING = 2;

m11RPGwj_SelectDialog.DEFAULT_FONT = "14px monospace";
m11RPGwj_SelectDialog.DEFAULT_ITEM_HEIGHT = 18;
