// タッチキーオブジェクト
var m11RPGwj_TouchKey = function(controller, key, pad_size, offset_x, offset_y) {

	this._controller = controller;
	this.key = key;

	this._pad_size = pad_size;

	// コントローラー内部の相対座標
	this._offset_x = offset_x;
	this._offset_y = offset_y;

	this._fore_rgb_on = "#66ccff";
	this._fore_rgb_off = "#336699";
	this._back_rgb_on = "#cccccc";
	this._back_rgb_off = "#999999";

	this.updateControllerPos(0, 0);

}

// コントローラー位置追随
m11RPGwj_TouchKey.prototype.updateControllerPos = function(x, y) {

	// キー位置設定
	this.left = x + this._offset_x;
	this.top = y + this._offset_y;
	this.right = this.left + this._pad_size;
	this.bottom = this.top + this._pad_size;

}

// (x, y)がキー領域内部か判定
m11RPGwj_TouchKey.prototype.isInner = function(x, y) {
	return (x > this.left) && (x < this.right) && (y > this.top) && (y < this.bottom);
}

// キー描画
m11RPGwj_TouchKey.prototype.draw = function(context) {

	// 背景クリア
	if ((this._controller.getKeyStatus() & this.key) == 0) {
		context.fillStyle = this._back_rgb_off;
	} else {
		context.fillStyle = this._back_rgb_on;
	}

	context.fillRect(this.left, this.top, this._pad_size, this._pad_size);

	// 内部図形描画色設定
	if ((this._controller.getKeyStatus() & this.key) == 0) {
		context.fillStyle = this._fore_rgb_off;
	} else {
		context.fillStyle = this._fore_rgb_on;
	}

	switch (this.key) {

	case m11RPGwj_Controller.KEYFLAG_SELECT:

		context.fillRect(this.left + this._pad_size / 4, this.top + this._pad_size / 4, this._pad_size / 2, this._pad_size / 2);

		break;

	case m11RPGwj_Controller.KEYFLAG_UP:

		context.beginPath();

		context.moveTo(this.left + this._pad_size / 4, this.top + 3 * this._pad_size / 4);
		context.lineTo(this.left + 3 * this._pad_size / 4, this.top + 3 * this._pad_size / 4);
		context.lineTo(this.left + this._pad_size / 2, this.top + this._pad_size / 4);

		context.closePath();

		context.fill();

		break;

	case m11RPGwj_Controller.KEYFLAG_RIGHT:

		context.beginPath();

		context.moveTo(this.left + this._pad_size / 4, this.top + this._pad_size / 4);
		context.lineTo(this.left + this._pad_size / 4, this.top + 3 * this._pad_size / 4);
		context.lineTo(this.left + 3 * this._pad_size / 4, this.top + this._pad_size / 2);

		context.closePath();

		context.fill();

		break;

	case m11RPGwj_Controller.KEYFLAG_DOWN:

		context.beginPath();

		context.moveTo(this.left + this._pad_size / 4, this.top + this._pad_size / 4);
		context.lineTo(this.left + 3 * this._pad_size / 4, this.top + this._pad_size / 4);
		context.lineTo(this.left + this._pad_size / 2, this.top + 3 * this._pad_size / 4);

		context.closePath();

		context.fill();

		break;

	case m11RPGwj_Controller.KEYFLAG_LEFT:

		context.beginPath();

		context.moveTo(this.left + 3 * this._pad_size / 4, this.top + this._pad_size / 4);
		context.lineTo(this.left + 3 * this._pad_size / 4, this.top + 3 * this._pad_size / 4);
		context.lineTo(this.left + this._pad_size / 4, this.top + this._pad_size / 2);

		context.closePath();

		context.fill();

		break;

	}

}

// タッチ操作コントローラーオブジェクト
var m11RPGwj_TouchController = function(controller) {

	// 親canvas要素の参照
	this._parent = null;

	// 親canvas要素のクライアント領域座標
	this._parent_x = 0;
	this._parent_y = 0;

	// 親canvas内座標
	this.left = 0;
	this.top = 0;
	this.right = 0;
	this.bottom = 0;

	this._controller = controller;

	// タッチ操作用キー配列
	this._keyList = new Array();

	// キー領域サイズ
	this._pad_size = 70;

	// コントローラー全体の大きさ
	this._width = this._pad_size * 3;
	this._height = this._pad_size * 5;

	// タッチ操作用キー作成
	this._keyList.push(new m11RPGwj_TouchKey(this._controller, m11RPGwj_Controller.KEYFLAG_SELECT, this._pad_size, this._pad_size, 0));

	this._keyList.push(new m11RPGwj_TouchKey(this._controller, m11RPGwj_Controller.KEYFLAG_UP, this._pad_size, this._pad_size, this._pad_size * 2));

	this._keyList.push(new m11RPGwj_TouchKey(this._controller, m11RPGwj_Controller.KEYFLAG_RIGHT, this._pad_size, this._pad_size * 2, this._pad_size * 3));

	this._keyList.push(new m11RPGwj_TouchKey(this._controller, m11RPGwj_Controller.KEYFLAG_DOWN, this._pad_size, this._pad_size, this._pad_size * 4));

	this._keyList.push(new m11RPGwj_TouchKey(this._controller, m11RPGwj_Controller.KEYFLAG_LEFT, this._pad_size, 0, this._pad_size * 3));

	this._key_flag = m11RPGwj_Controller.KEYFLAG_NONE;

	// (x, y)がコントローラー領域内部か判定
	this.isInner = function(x, y) {
		return (x > this.left) && (x < this.right) && (y > this.top) && (y < this.bottom);
	}

	this.getWidth = function() {
		return this._width;
	}

	this.getHeight = function() {
		return this._height;
	}

	// キー状態を取得
	this.getKeyStatus = function() {
		return this._key_flag;
	}

	// 方向キー状態取得
	this.getStick = function() {
		return this._key_flag & m11RPGwj_Controller.STICK_MASK;
	}

	// ボタンキー状態取得
	this.getButtons = function() {
		return this._key_flag & m11RPGwj_Controller.BUTTONS_MASK;
	}

	// 親canvas設定
	this.setParent = function(cv) {
		this._parent = cv;
	}

	this.syncPos = function() {

		var parentRect = this._parent.getBoundingClientRect();

		// 親canvasのブラウザクライアント領域内座標記録
		this._parent_x = parentRect.left;
		this._parent_y = parentRect.top;

	}

	// 親canvas上での位置設定
	this.setPos = function(x, y) {

		var i;

		// 親canvas内の相対座標設定
		this.left = x;
		this.top = y;
		this.right = this.left + this._width;
		this.bottom = this.top +  + this._height;

		// 各キーの位置更新
		for (i = 0;i < this._keyList.length;i++) {
			this._keyList[i].updateControllerPos(this.left, this.top);
		}

	}

	this.getX = function() {
		return this.left;
	}

	this.getY = function() {
		return this.top;
	}

	// タッチイベント処理
	this.eventHandler = function(e) {

		var i;

		// コントローラーの位置を更新
		this.syncPos();

		this._key_flag = m11RPGwj_Controller.KEYFLAG_NONE;

		switch (e.type) {

		// タッチ操作
		case "touchstart":
		case "touchmove":

			var x = e.touches[0].clientX - this._parent_x;
			var y = e.touches[0].clientY - this._parent_y;

			if (this.isInner(x, y)) {
				e.preventDefault();
			}

			// 各キーのタッチ判定を行いキー状態反映
			for (i = 0;i < this._keyList.length;i++) {
				if (this._keyList[i].isInner(x, y)) {
					this._key_flag |= this._keyList[i].key;
				}
			}

			break;

		}

	}

	this.draw = function() {

		var context = this._parent.getContext("2d");

		var i;

		var prev_alpha = context.globalAlpha;

		context.globalAlpha = 0.6;

		// 各キーの描画処理
		for (i = 0;i < this._keyList.length;i++) {
			this._keyList[i].draw(context);
		}

		context.globalAlpha = prev_alpha;

	}

}

